package dao

import (
	"github.com/jinzhu/gorm"
	"time"
)

type LightStrategy struct {
	ID                 int       `gorm:"primary key" json:"id"`                        //编号
	LightName          string    `gorm:"type: varchar(64" json:"lightName"`            //策略名称
	LightControlType   int       `gorm:"type:int" json:"lightControlType"`             //灯控类型 设置 1=485,2=nb-iot
	OptoSensorID       int       `gorm:"type:int" json:"optoSensorID"`                 //光传感器ID
	StartTime          string    `gorm:"type:date" json:"startTime"`                   //策略期限开始时间
	EndTime            string    `gorm:"type:date" json:"endTime"`                     //策略期限结束时间
	IsAllYear          int       `gorm:"type:int;default 1" json:"isAllYear"`          //全年设置2=是，1=否
	IsAutomaticRenewal int       `gorm:"type:int;default 1" json:"isAutomaticRenewal"` //自动续期设置 2=是，1=否
	CreateTime         time.Time `gorm:"type: datetime" json:"createTime"`             //新增时间
	CreateUser         string    `gorm:"type:varchar(60)" json:"createUser"`           //新增记录操作用户ID
	UpdateTime         time.Time `gorm:"type: datetime" json:"updateTime"`             //修改时间
	UpdateUser         string    `gorm:"type:varchar(60)" json:"updateUser"`           //修改用户
	IsDeleted          int       `gorm:"type:int;default 0" json:"isDeleted"`          //是否删除 0=未删除,1=删除
	Status             int       `gorm:"type:int;default 1" json:"status"`             //状态 0=正常,1=异常
	TenantID           string    `gorm:"type:varchar(12)" json:"tenantID"`             //租户id
	LightSn            string    `gorm:"type:varchar(64" json:"lightSn"`               //策略编码
	ManualTime         int       `gorm:"type:int;default 0" json:"manualTime"`         //手动控制时间
}

func (LightStrategy) TableName() string {
	return "t_strategy_light"
}

func (c *LightStrategy) Delete() error {
	return Db.Model(&c).Where("id = ?", c.ID).Updates(map[string]interface{}{"update_time": c.UpdateTime,
		"update_user": c.UpdateUser, "is_deleted": c.IsDeleted}).Error
}

func (c LightStrategy) IsExistedBySN() bool {
	var count = 0
	_ = Db.Model(&c).Where(" light_sn = ? and is_deleted = ?",
		c.LightSn, c.IsDeleted).Count(&count).Error
	return count > 0
}

func (c LightStrategy) IsExistedByNameAndCode() bool {
	var Strategys []LightStrategy
	err := Db.Model(&c).Where("gateway_sn = ? and is_deleted = ?",
		c.TenantID, c.IsDeleted).Find(&Strategys).Error
	if gorm.IsRecordNotFoundError(err) {
		return false
	}
	for _, d := range Strategys {
		if d.ID != c.ID {
			return true
		}
	}
	return false
}

func (c *LightStrategy) Create() error {
	return Db.Model(&c).Save(&c).Error
}

func (c *LightStrategy) Update() error {
	return Db.Model(&c).Where(" id = ? ", c.ID).Update(&c).Error
}

func (c *LightStrategy) GetStrategy() error {
	err := Db.Model(&c).Where(" id = ? ", c.ID).First(&c).Error
	return err
}

func (c LightStrategy) GetStrategies(offset, limit int) ([]LightStrategy, error) {
	var Strategys []LightStrategy
	err := Db.Model(&c).Where(" name like ? and is_deleted = 0", "%"+c.LightName+"%").Offset(offset).Limit(limit).Find(&Strategys).Error
	return Strategys, err

}

func (c LightStrategy) GetAllStrategies() ([]*LightStrategy, error) {
	var Strategys []*LightStrategy
	err := Db.Model(&c).Where(" tenant_id = ? and is_deleted = ? ", c.TenantID, c.IsDeleted).Scan(&Strategys).Error
	return Strategys, err
}
